// WP uses a 'no conflict' version of jQuery; let's
// grab a reference to the 'jQuery' function
var $j = jQuery.noConflict();
$j('document').ready(function()
{

	var WT_RMRH_WPPLUGIN = 
	{
		/**
		 * Error message displayed when there is a problem retrieving the post content
		 */
		ERROR_MSG : "Sorry! There was an error retrieving content.<br>Please click link again to be taken to this entry's page.",
		
		
		/**
		 * CSS stylings applied to the error message 
		 */
		ERROR_STYLE : {	'color':'#8C0913',
						'font-weight':'bold',
						'backgroundColor':'#ccc',
						'border':'thin solid #8C0913',
						'padding':'15px',
						'text-align':'center'},
						
		/**
		 * GIF "loading" image
		 */
		LOADING_IMAGE :  
			$j(new Image())
				.attr('src', RMRH_VALUES.pluginUrl + RMRH_VALUES.loadImageName)
				.css('margin-left', '10px'),
		
		/**
		 * Boolean indicating if we will dupliacte the 'more link' to the bottom
		 * 	of the post.
		 */
		DUPLICATE_LINK : (parseInt(RMRH_VALUES.duplicateLink) == 1),
			
		/**
		 * Text used for the 'more link' after content has expanded
		 */
		EXPANDED_TEXT : RMRH_VALUES.expandedText,
		
		/**
		 * Boolean indicating if we should use new text for 'more link'
		 * 	after content has expanded
		 */
		USE_EXPANDED_TEXT : !(typeof RMRH_VALUES.expandedText == 'undefined'),
		
		/**
		 * Speed of 'slide down' animation for new post content.
		 */
		ANIMATE_SPEED : parseInt(RMRH_VALUES.animateSpeed),
		
		
		/**
		 *	Returns the ID of the post we want by parsing the 
		 *	 special class name inserted into the 'more' link
		 *	 by the 'the_content_more_link' Wordpress filter.
		 *	<br>
		 *	@param the anchor element 'more' link
		 *	@return the target post id
		 */
		getPostId : function(anchorEl)
		{
			var classNames = anchorEl.attr("class"),
				matches = classNames.match("rmrh-postId-([0-9]+)\\b");
			
			if(matches.length > 0)
			{
				// also going to save the found class name to
				//	internal element data
				anchorEl.data('rmrhClassName',matches[0]);
				
				// and an id for the clone if one's needed
				anchorEl.data('rmrhLinkCloneId',matches[0]+'-clone-'+(new Date()).getTime());
				
				return matches[1];
			}
			else
			{
				return null;
			}
				
		},
		
		/**
		 *	Add our ajax request action to the 'click' event
		 *	of the 'more' anchor element.
		 *	<br>
		 *	@param Element the 'more' anchor element
		 */
		setRedirectRequest : function(el)
		{
			var url = el.attr('href'),
				postId = this.getPostId(el);
			
			// If not able to retrieve a post ID, there
			//  is nothing else we can do. Just leave
			//	everything alone.
			if(postId == null)
			{
				return;
			}
			
			el.bind('click', 
					{"el":el, "url":url, "postid":postId}, 
				 	this.ajaxClick);
		},
	
		/**
		 *	The 'more' element's 'click' event handler
		 *	<br>
		 *	@param event data set as part of 
		 *		   <code>setRedirectRequest</code>
		 */
		ajaxClick : function(event)
		{
			var 
				theEl = event.data.el,
				theImg = WT_RMRH_WPPLUGIN.LOADING_IMAGE.clone();
			
			// append and display the loading image 
			// after the 'more' anchor element
			theEl.after(theImg);
			theImg.show();
			
			// perform the ajax request
			$j.ajax
			({
				type: "POST",
				url: event.data.url,
				dataType: "html",
				cache: false,
				data: {'wt-rmrh-redirect':'1', 'postid':event.data.postid},
				error: function(request, textStatus, errorThrown)
				{
					data = WT_RMRH_WPPLUGIN.ERROR_MSG;
					WT_RMRH_WPPLUGIN.ajaxFinished(theEl, theImg, data, true);
				},
				success: function(data, textStatus)
				{
					WT_RMRH_WPPLUGIN.ajaxFinished(theEl, theImg, data, false);
				}
	
			});
			// keep anchor 'click' event propagating
			return false; 
		},
		
		/**
		 *	Handles the completion of our ajax request. If the
		 *	request resulted in an error, the 'more' anchor 
		 *	element will revert to its normal use (i.e. click
		 *	results in loading the post's single page display).
		 *	<br>
		 *	@param Element 'more' anchor element
		 *	@param Image loading image
		 *	@param String ajax request result data
		 *	@param boolean true if request resulted in error; 
		 *		   false otherwise 
		 */
		ajaxFinished : function(theEl, theImg, result, bError)
		{
			var newEl = $j("<p>").html(result).hide(),
				origAnchorText = theEl.html(),
				
				showLoadingImageFunc = 
					function()
					{
						theEl.after(theImg);
						theImg.show();
					},
					
				hideLoadingImageFunc =
					function()
					{
						theImg.fadeOut(WT_RMRH_WPPLUGIN.ANIMATE_SPEED/2);
					},
					
				cloneClickFunc =
					function()
					{
						var scrollTo = theEl.offset().top - 200;
						scrollTo = (scrollTo <= 0) ? 0 : scrollTo;
						
						// If the 'more link' is not still visible, scroll back to it
						//	(checking distance to travel needed, and if less than what
						//	is visible in the window, don't animate)
						if(($j(this).offset().top - scrollTo) > $j(window).height())
							$j('html,body').animate({scrollTop:(scrollTo)+"px"},WT_RMRH_WPPLUGIN.ANIMATE_SPEED/2);
						
						// wait for scroll back to finish, then trigger a 'click' (which
						//	will cause a collapse)
						setTimeout(function(){theEl.trigger('click')},WT_RMRH_WPPLUGIN.ANIMATE_SPEED/2);
						
						return false;	
					},
					
				moreLinkClone = 
					theEl.clone().click(cloneClickFunc).attr('id',theEl.data('rmrhLinkCloneId')),
					
				expandFunc =
					function()
					{
						showLoadingImageFunc();
						
						// When 'expanding', want any 'object' elements
						//	to show last.
						newEl.slideDown(WT_RMRH_WPPLUGIN.ANIMATE_SPEED, function()
								{ 
									hideLoadingImageFunc();
									newEl.find('object').show();
									
									// Want to change the 'more link' text?
									if(WT_RMRH_WPPLUGIN.USE_EXPANDED_TEXT)
									{
										theEl.html(WT_RMRH_WPPLUGIN.EXPANDED_TEXT);
									}
									
									// Want 'more link' duplicate at bottom?
									if(WT_RMRH_WPPLUGIN.DUPLICATE_LINK)
									{
										if($j('#'+moreLinkClone.attr('id')).size() == 0)
										{
											// for use when jQuery updated in WP release
											//theEl.siblings().last().after(moreLinkClone);
											var sibs = theEl.siblings();
											$j(sibs[sibs.size()-1]).after(moreLinkClone);
										}
										
										moreLinkClone.html(theEl.html()).show();
									}
								});
								
						return false;	
					},
					
				collapseFunc =
					function()
					{
						showLoadingImageFunc();
						
						// If created a duplicate link at the bottom,
						//	hide it
						if(WT_RMRH_WPPLUGIN.DUPLICATE_LINK)
						{
							$j('#'+theEl.data('rmrhLinkCloneId')).hide();
						}
						
						// When collapsing, want any 'object' elements
						//	to hide first.
						newEl.find('object').hide();		
						newEl.slideUp(WT_RMRH_WPPLUGIN.ANIMATE_SPEED, function()
								{ 
									hideLoadingImageFunc();
									
									// If use special expanded text, replace it
									//	with the original
									if(WT_RMRH_WPPLUGIN.USE_EXPANDED_TEXT)
									{
										theEl.html(origAnchorText);
									}
								});	
								
						
							
						return false;
					},
					
		
				/**
				 * Does the given element contain an 'object' element.
				 */
				hasEmbed = function(el)
				{
					var result = false;
					el.find('object').each(
							function()
							{	
								result = true;
								return;
							});
					
					return result;
				};
			
			// AJAX request finished, don't want it to run again
			theEl.unbind('click', WT_RMRH_WPPLUGIN.ajaxClick);
			
			// If IE 7 and newer, and the new content has an 
			//	embedded object (e.g. flash video), we have 
			//	to just re-direct to the single page entry.
			//	The object will NOT display.
			if($j.browser.msie && (parseInt($j.browser.version) > 6))
			{
				if(hasEmbed(newEl))
				{
					window.location = theEl.attr('href');
					return;
				}
			}
			
			// All 'object' elements are initially hidden
			newEl.find('object').hide();
	
			// the loading gif is after the more link, so
			// put the new content after that image
			theImg.after(newEl);
	
	
			// if no error, 'more' link slides the content in and
			// out of view; otherwise future clicks behave normally
			// and take user to the single post page
			if(!bError)
			{
				theEl.toggle
				(
					expandFunc,
					collapseFunc
				);
				
				// trigger a 'click' for the initial content slide down
				theEl.click();
			}
			else
			{
				// have an error: set error styles on the new element and show it
				newEl.css(WT_RMRH_WPPLUGIN.ERROR_STYLE);
				newEl.slideDown(WT_RMRH_WPPLUGIN.ANIMATE_SPEED,function(){theImg.fadeOut(WT_RMRH_WPPLUGIN.ANIMATE_SPEED/2)});
			}
	
		}
	};


	// 'the_content_more_link' filter added a class to the anchor with the
	//	post ID included.  Find all anchors matching the pattern 'rmrh-postId-[id]'
	$j("a[class^=rmrh-postId-]").each
	(
		function(index)
		{	
			WT_RMRH_WPPLUGIN.setRedirectRequest($j(this));			
		}
	);
		
});